from vital_ai_vitalsigns.impl import VitalSignsImpl
from vital_ai_vitalsigns.model.trait.PropertyTrait import PropertyTrait


class VITALNode:
    allowed_properties = []

    def __init__(self):
        super().__setattr__('_properties', {})

    def __setattr__(self, name, value):
        print(name)
        for prop_info in self.allowed_properties:
            uri = prop_info['uri']
            prop_class = prop_info['prop_class']
            trait_class = VitalSignsImpl.get_trait_class_from_uri(uri)

            if trait_class and trait_class.get_short_name() == name:
                if value is None:
                    self._properties.pop(uri, None)
                else:
                    self._properties[uri] = VitalSignsImpl.create_property_with_trait(prop_class, uri, value)
                return

        raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")

    def __getattr__(self, name):
        for prop_info in self.allowed_properties:
            uri = prop_info['uri']
            # print(uri)
            trait_class = VitalSignsImpl.get_trait_class_from_uri(uri)
            # print(trait_class)
            if trait_class and trait_class.get_short_name() == name:
                if uri in self._properties:
                    return self._properties[uri]
                else:
                    return None
        raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")




from datetime import datetime
from .IProperty import IProperty


class DateTimeProperty(IProperty):
    def __init__(self, value: datetime):
        super().__init__(value)

    def __str__(self):
        return self.value.strftime('%Y-%m-%d')



from vital_ai_vitalsigns.model.VITAL_Node import VITALNode
from vital_ai_vitalsigns.model.properties.DateTimeProperty import DateTimeProperty
from vital_ai_vitalsigns.model.properties.StringProperty import StringProperty


class Person(VITALNode):
    allowed_properties = [
        {'uri': "http://vital.ai/ontology/vital-core#hasName", 'prop_class': StringProperty},
        {'uri': "http://vital.ai/ontology/vital-core#hasBirthday", 'prop_class': DateTimeProperty}
    ]




from vital_ai_vitalsigns.model.trait.PropertyTrait import PropertyTrait
import pkgutil
import importlib


class VitalSignsImpl:

    @staticmethod
    def import_submodules(package_name):
        try:
            package = importlib.import_module(package_name)
            package_path = package.__path__
            for loader, name, is_pkg in pkgutil.walk_packages(path=package_path):
                full_name = f"{package_name}.{name}"
                # print(full_name)
                try:
                    importlib.import_module(full_name)
                    if is_pkg:
                        VitalSignsImpl.import_submodules(full_name)
                except ImportError as e:
                    pass  # print(f"Error importing {full_name}: {e}")
        except ImportError as e:
            pass  # print(f"Error importing package {package_name}: {e}")
    @staticmethod
    def find_all_subclasses(base_class):
        return base_class.__subclasses__()

    @staticmethod
    def create_property_with_trait(property_class, trait_uri, value):

        VitalSignsImpl.import_submodules('vital_ai_vitalsigns_core')
        VitalSignsImpl.import_submodules('vital_ai_vitalsigns')

        trait_classes = [cls for cls in PropertyTrait.__subclasses__() if cls.get_uri() == trait_uri]

        if not trait_classes:
            raise ValueError(f"No trait found with URI: {trait_uri}")

        trait_class = trait_classes[0]

        class CombinedProperty(property_class, trait_class):
            def get_uri(self):
                return super().get_uri()

        return CombinedProperty(value)

    @staticmethod
    def get_trait_class_from_uri(uri):

        VitalSignsImpl.import_submodules('vital_ai_vitalsigns_core')

        for subclass in PropertyTrait.__subclasses__():
            if subclass.get_uri() == uri:
                return subclass
        return None
